import pandas as pd
from pandas import DataFrame as df
from datetime import datetime

import psycopg2 as pg
import sqlalchemy as sa
#import ibm_db
#import ibm_db_sa
import cx_Oracle as co
import pymysql


def postgreSQL(query, h=None, port=None, db=None, u=None, p=None):

	print('Using PostgreSQL')

	# DB Connection
	conn = pg.connect(host=h,
			  port=str(port),
			  user=u,
			  password=p)
	start_tm = datetime.now()
	print(' Start :', str(start_tm))

	# Get a DataFrame
	query_result = pd.read_sql(query, conn)

	# Cloase Connection
	end_tm = datetime.now()
	print(' Finish :', str(end_tm))
	print('Elapsed :', str(end_tm - start_tm))
	conn.close()

	return query_result


def db2SQL(query, h=None, port=None, db=None, u=None, p=None):

 	print('Using IBM DB2')
	
	# DB Connection
	connStr = 'ibm_db_sa://{}:{}@{}:{}/{}'
	engine = sa.create_engine(connStr.format(u, p, h, str(port), db))
	conn = engine.connect()
	start_tm = datetime.now()
	print(' Start :', str(start_tm))

	# Get a DataFrame
	execonn = engine.execute(query)

	query_result = df(execonn.fetchall())
	euqry_result.columns = execonn.keys()

	# Close Connection
	end_tm = datetime.now()
	print(' Finish :', str(end_tm))
	print('Elapsed :', str(end_tm - start_tm))
	conn.close()

	return query_result

def oracleSQL(query, h=None, port=None, db=None, u=None, p=None):
	
	print('Using OracleDB')

	# DB Connection
	dnsStr = co.makedsn(h, str(port), db))
	dnsStr = dnsStr.replace('SID', 'SERVICE_NAME')
	conn = co.connect(u, p, dnsStr)

	# Get a DataFrame
	query_result = pd.read_sql(query, conn)

	# Close Connection
	end_tm = datetime.now()
	print(' Finish :', str(end_tm))
	print('Elapsed :', str(end_tm - start_tm))
	conn.close()

	return query_result


def mariaDB(query, h=None, port=None, db=None, u=None, p=None):
	
	print('Using MariaDB')
	
	# DB Connection
	conn = pymysql.connect(host=h, port=p, user=u, password=p, database=db)	
	start_tm = datetime.now()
	print(' Start :', str(start_tm))

	# Get a DataFrame
	query_result = pd.read_sql(query, conn)

	# Close Connection
	end_tm = datetime.now()
	print(' Finish :', str(end_tm))
	print('Elapsed :', str(end_tm - start_tm))
	conn.close()

	return query_result






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan  3 23:18:27 2017

@author: dawkiny
"""

import numpy as np
import pandas as pd
from pandas import DataFrame as df
from datetime import datetime as dt
import itertools as it
import random, string
import matplotlib.pyplot as plt

###########################
fab_id='M14'
lot_cd='TCC'
alias_lot_id='TCCK108'
wf_id=list(range(1, 26))
end_tm = '20161101180025'
size=(40, 30)
############################

# Create Wafer DataFrame
def create(fab_id=None, lot_cd=None, end_tm=None, size=(None, None)):
    
    xsize, ysize = size
    
    wafer = df(columns=['fab_id', 'lot_cd', 'alias_lot_id', 'wf_id',
                        'end_dt', 'end_tm', 'xcoord', 'ycoord', 'res_val'],
               index=range(0, xsize * ysize * 25))
    
    # Import values to each Field
    
    ## fab_id, lot_cd
    wafer.fab_id, wafer.lot_cd = fab_id, lot_cd
    
    ## alias_lot_id
    id1 = ''.join([random.choice(string.ascii_uppercase + string.digits) for n in range(1)])
    id2 = ''.join([random.choice(string.digits) for n in range(3)])
    wafer.alias_lot_id = lot_cd + id1 + id2
    
    ## wf_id
    wf_iter = [list(it.repeat(wf_num, xsize * ysize)) for wf_num in range(1, 26)]
    wafer.wf_id = list(it.chain.from_iterable(wf_iter))
    wafer.wf_id = wafer.wf_id.astype(str).str.zfill(2)
    
    ## end_tm, end_dt
    wafer.end_tm = dt.strptime(end_tm, '%Y%m%d%H%M%S')
    wafer.end_dt = end_tm.date()
    
    ## x_coordinate, y_coordinate
    coords = tuple(it.product(list(range(1, xsize+1)), list(range(1, ysize+1)) ))
    wafer.xcoord, wafer.ycoord = zip(*coords*25)
    
    ## res_val
    wafer.res_val = np.random.choice(list(string.ascii_uppercase[0:20]), wafer.shape[0])
    
    print('-'*25)
    print('ALIAS_LOT_ID : ' + lot_cd + id1 + id2)
    print('-'*25)
    return wafer
    


# Wafer Visualization
def plot():
    plt.figure(figsize=(12, 9))
    plt.scatter(wafer.xcoord, wafer.ycoord, marker='+', s=1000, c='grey', alpha=.1)
    plt.scatter(wafer.xcoord[wafer.res_val=='A'], wafer.ycoord[wafer.res_val=='A'], marker='s', s=100, c='red', alpha=.5)
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan  3 23:18:27 2017

@author: dawkiny
"""

import numpy as np
from pandas import DataFrame as df
from datetime import datetime as dt
import itertools as it
import random, string
import matplotlib
import matplotlib.pyplot as plt


from pydawkiny.geometry import Ellipse


class wafer:
    
    # Create lotData DataFrame
    def __init__(self, fac_id=None, lot_cd=None, end_tm=None, size=None, unit_cnt=25, pattern=['G', 'B'], p=[.8, .2]):

        assert len(size) == 2
        assert len(pattern) == len(p)
        assert sum(p) == 1
        assert isinstance(unit_cnt, int) == True
        self.fac_id = fac_id
        self.lot_cd = lot_cd
        self.end_tm = end_tm
        self.size = size
        self.unit_cnt = unit_cnt
        self.failrate = p[0]
        
    def __new__(cls, fac_id=None, lot_cd=None, end_tm=None, size=None, unit_cnt=25, pattern=['G', 'B'], p=[.8, .2]):
        
        coords = Ellipse(size).coordinates()
        #xsize, ysize = coords.xcoord.max(), coords.ycoord.max()
        
        lotData = df(columns=['fac_id', 'lot_cd', 'lot_id', 'unit_id',
                            'end_dt', 'end_tm', 'xcoord', 'ycoord', 'val'],
                   index=range(0, coords.shape[0] * unit_cnt))

        # Import values to each Field
        
        ## fab_id, lot_cd
        lotData.fac_id, lotData.lot_cd = fac_id, lot_cd
        
        ## alias_lot_id
        id1 = ''.join([random.choice(string.ascii_uppercase + string.digits) for n in range(1)])
        id2 = ''.join([random.choice(string.digits) for n in range(3)])
        lotData.lot_id = lot_cd + id1 + id2
        
        ## wf_id
        unitIter = [list(it.repeat(num, len(coords))) for num in range(1, unit_cnt+1)]
        lotData.unit_id = list(it.chain.from_iterable(unitIter))
        lotData.unit_id = lotData.unit_id.astype(str).str.zfill(2)
        
        ## end_tm, end_dt
        lotData.end_tm = dt.strptime(end_tm, '%Y%m%d%H%M%S')
        lotData.end_dt = lotData.end_tm.map(lambda x: x.date())
        
        ## x_coordinate, y_coordinate
        lotData.xcoord = list(coords.xcoord) * unit_cnt
        lotData.ycoord = list(coords.ycoord) * unit_cnt
        
        ## res_val
        failList = [ np.random.choice(pattern, size=len(coords), p=p) for num in range(1, unit_cnt+1)]
        lotData.val = list(it.chain.from_iterable(failList))
        
        
        # Print a message in console
        msg = 'LOT_ID : ' + lot_cd + id1 + id2
        print('-' * len(msg))
        print(msg)
        print('-' * len(msg))
        return lotData

        
    # lotData Visualization
    def unitplot(data, columns=['xcoord', 'ycoord'], lot_id='', unit_id=None, val='val', figsize=(12, 9), pattern=['B'], m=['s'], s=[100], c=['red'], a=[.5], shown='y'):
        
        assert len(figsize) == 2
        assert len(columns) == 2
        assert isinstance(lot_id, str) == True
        assert len(pattern) == len(m) == len(s) == len(c) == len(a)
        
        
        # Check if there is a given Wafer ID
        if unit_id != None:
            pltData = data.loc[data.unit_id==unit_id, [columns[0], columns[1], val]]
        else:
            pltData = data.loc[:, [columns[0], columns[1], val]]
            unit_id = ''

        
        # Decide if the plot is shown
#        if shown == 'y':
#            matplotlib.use('Agg')
#            plt.ion()
#    
#        elif shown == 'n':
#            # Turn interactive plotting off
#            matplotlib.use('Agg')
#            plt.ioff()
        
        canvas = {'pattern':'+', 's':1000, 'c':'grey', 'a':.2}

        # Draw plot
        fig = plt.figure(figsize=figsize)
        plt.scatter(pltData.xcoord, pltData.ycoord, marker=canvas['pattern'], s=canvas['s'], c=canvas['c'], alpha=canvas['a'])
        for idx in range(0, len(pattern)):
            plt.scatter(pltData.loc[pltData.val==pattern[idx], 'xcoord'],
                        pltData.loc[pltData.val==pattern[idx], 'ycoord'],
                        marker=m[idx], s=s[idx], c=c[idx], alpha=a[idx])

        fig.suptitle(str(lot_id + unit_id))
        #fig.tight_layout()
        
        if shown == 'n':
            plt.close(fig)

        return fig
        


        
class displaypanel(wafer):
    
    # Create lotData DataFrame
    def __init__(self, fac_id=None, lot_cd=None, end_tm=None, size=None, unit_cnt=25, pattern=['G', 'B'], p=[.8, .2]):

        assert len(size) == 2
        assert len(pattern) == len(p)
        assert sum(p) == 1
        assert isinstance(unit_cnt, int) == True
        self.fac_id = fac_id
        self.lot_cd = lot_cd
        self.end_tm = end_tm
        self.size = size
        self.unit_cnt = unit_cnt
        self.failrate = p[0]
        
    def __new__(cls, fac_id=None, lot_cd=None, end_tm=None, size=None, unit_cnt=25, pattern=['G', 'B'], p=[.8, .2]):
        
        xsize, ysize = size
        
        lotData = df(columns=['fac_id', 'lot_cd', 'lot_id', 'unit_id',
                            'end_dt', 'end_tm', 'xcoord', 'ycoord', 'val'],
                   index=range(0, xsize * ysize * unit_cnt))

        # Import values to each Field
        
        ## fab_id, lot_cd
        lotData.fac_id, lotData.lot_cd = fac_id, lot_cd
        
        ## alias_lot_id
        id1 = ''.join([random.choice(string.ascii_uppercase + string.digits) for n in range(1)])
        id2 = ''.join([random.choice(string.digits) for n in range(3)])
        lotData.lot_id = lot_cd + id1 + id2
        
        ## pn_id
        unitIter = [list(it.repeat(num, xsize * ysize)) for num in range(1, unit_cnt+1)]
        lotData.unit_id = list(it.chain.from_iterable(unitIter))
        lotData.unit_id = lotData.unit_id.astype(str).str.zfill(2)
        
        ## end_tm, end_dt
        lotData.end_tm = dt.strptime(end_tm, '%Y%m%d%H%M%S')
        lotData.end_dt = lotData.end_tm.map(lambda x: x.date())
        
        ## x_coordinate, y_coordinate
        coords = tuple(it.product(list(range(1, xsize+1)), list(range(1, ysize+1)) ))
        lotData.xcoord, lotData.ycoord = zip(*coords*unit_cnt)
        
        ## res_val
        failList = [ np.random.choice(pattern, size=len(coords), p=p) for num in range(1, unit_cnt+1)]
        lotData.val = list(it.chain.from_iterable(failList))
        
        # Print a message in console
        msg = 'LOT_ID : ' + lot_cd + id1 + id2
        print('-' * len(msg))
        print(msg)
        print('-' * len(msg))
        return lotData


    
        
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan  3 23:18:27 2017

@author: dawkiny
"""

import numpy as np
import pandas as pd
from pandas import DataFrame as df
from datetime import datetime as dt
import itertools as it
import random, string
import matplotlib.pyplot as plt

import math
from pydawkiny.geometry import Ellipse


class lot:
    
    # Create lotData DataFrame
    def __init__(self, fab_id=None, lot_cd=None, end_tm=None, size=None):

        assert len(size) == 2
        self.fab_id = fab_id
        self.lot_cd = lot_cd
        self.end_tm = end_tm
        self.size = size
        
    def __new__(cls, fab_id=None, lot_cd=None, end_tm=None, size=None):
        
        coords = Ellipse(size).coordinates()
        xsize, ysize = coords.xcoord.max(), coords.ycoord.max()
        
        lotData = df(columns=['fab_id', 'lot_cd', 'alias_lot_id', 'wf_id',
                            'end_dt', 'end_tm', 'xcoord', 'ycoord', 'res_val'],
                   index=range(0, coords.shape[0] * 25))

        # Import values to each Field
        
        ## fab_id, lot_cd
        lotData.fab_id, lotData.lot_cd = fab_id, lot_cd
        
        ## alias_lot_id
        id1 = ''.join([random.choice(string.ascii_uppercase + string.digits) for n in range(1)])
        id2 = ''.join([random.choice(string.digits) for n in range(3)])
        lotData.alias_lot_id = lot_cd + id1 + id2
        
        ## wf_id
        lotIter = [list(it.repeat(wf_num, len(coords))) for wf_num in range(1, 26)]
        lotData.wf_id = list(it.chain.from_iterable(lotIter))
        lotData.wf_id = lotData.wf_id.astype(str).str.zfill(2)
        
        ## end_tm, end_dt
        lotData.end_tm = dt.strptime(end_tm, '%Y%m%d%H%M%S')
        lotData.end_dt = lotData.end_tm.map(lambda x: x.date())
        
        ## x_coordinate, y_coordinate
        lotData.xcoord = list(coords.xcoord) * 25
        lotData.ycoord = list(coords.ycoord) * 25
        
        ## res_val
        lotData.res_val = np.random.choice(list(string.ascii_uppercase[0:20]), lotData.shape[0])
        
        print('-'*25)
        print('ALIAS_LOT_ID : ' + lot_cd + id1 + id2)
        print('-'*25)
        return lotData
        
        
        return super().__new__(cls)
        
    def create(self):

        coords = Ellipse(self.size).coordinates()
        xsize, ysize = coords.xcoord.max(), coords.ycoord.max()
        
        lotData = df(columns=['fab_id', 'lot_cd', 'alias_lot_id', 'wf_id',
                            'end_dt', 'end_tm', 'xcoord', 'ycoord', 'res_val'],
                   index=range(0, coords.shape[0] * 25))

        # Import values to each Field
        
        ## fab_id, lot_cd
        lotData.fab_id, lotData.lot_cd = self.fab_id, self.lot_cd
        
        ## alias_lot_id
        id1 = ''.join([random.choice(string.ascii_uppercase + string.digits) for n in range(1)])
        id2 = ''.join([random.choice(string.digits) for n in range(3)])
        lotData.alias_lot_id = self.lot_cd + id1 + id2
        
        ## wf_id
        lotIter = [list(it.repeat(wf_num, len(coords))) for wf_num in range(1, 26)]
        lotData.wf_id = list(it.chain.from_iterable(lotIter))
        lotData.wf_id = lotData.wf_id.astype(str).str.zfill(2)
        
        ## end_tm, end_dt
        lotData.end_tm = dt.strptime(self.end_tm, '%Y%m%d%H%M%S')
        lotData.end_dt = lotData.end_tm.map(lambda x: x.date())
        
        ## x_coordinate, y_coordinate
        lotData.xcoord = list(coords.xcoord) * 25
        lotData.ycoord = list(coords.ycoord) * 25
        
        ## res_val
        lotData.res_val = np.random.choice(list(string.ascii_uppercase[0:20]), lotData.shape[0])
        
        print('-'*25)
        print('ALIAS_LOT_ID : ' + self.lot_cd + id1 + id2)
        print('-'*25)
        return lotData

        
    # lotData Visualization
    def wfplot(data, columns=['xcoord', 'ycoord'], wfid=None, figsize=None, bad=None, m=('+', 's'), s=(1000, 100), c=('grey', 'red'), a=(.1, .5), shown=None):
        
        assert len(figsize) == 2
        assert len(columns) == 2
        assert isinstance(wfid, str) == True
        assert len(wfid) == 2
        
        if shown == 'y':
            
            if wfid != None:

                pltData = data[data.wf_id==wfid]
            
                fig = plt.figure(figsize=figsize)
                plt.scatter(pltData.xcoord, pltData.ycoord, marker=m[0], s=s[0], c=c[0], alpha=a[0])
                plt.scatter(pltData.xcoord[pltData.res_val==bad],
                            pltData.ycoord[pltData.res_val==bad], marker=m[1], s=s[1], c=c[1], alpha=a[1])
                fig.suptitle(str(pltData.alias_lot_id.unique() + pltData.wf_id.unique()))
                #fig.tight_layout()
    
            else:
    
                pltData = data
                pltData.xcoord = pltData[columns[0]]
                pltData.ycoord = pltData[columns[1]]
            
                fig = plt.figure(figsize=figsize)
                plt.scatter(pltData.xcoord, pltData.ycoord, marker=m[0], s=s[0], c=c[0], alpha=a[0])
                plt.scatter(pltData.xcoord[pltData.res_val==bad],
                            pltData.ycoord[pltData.res_val==bad], marker=m[1], s=s[1], c=c[1], alpha=a[1])
                fig.suptitle(str(pltData.alias_lot_id.unique() + pltData.wf_id.unique()))
                #fig.tight_layout()
        
    elif shown == 'n':
        
        if wfid != None:

            pltData = data[data.wf_id==wfid]
        
            fig = plt.figure(figsize=figsize)
            fig.scatter(pltData.xcoord, pltData.ycoord, marker=m[0], s=s[0], c=c[0], alpha=a[0])
            fig.scatter(pltData.xcoord[pltData.res_val==bad],
                        pltData.ycoord[pltData.res_val==bad], marker=m[1], s=s[1], c=c[1], alpha=a[1])
            fig.suptitle(str(pltData.alias_lot_id.unique() + pltData.wf_id.unique()))
            #fig.tight_layout()

        else:

            pltData = data
            pltData.xcoord = pltData[columns[0]]
            pltData.ycoord = pltData[columns[1]]
        
            fig = plt.figure(figsize=figsize)
            fig.scatter(pltData.xcoord, pltData.ycoord, marker=m[0], s=s[0], c=c[0], alpha=a[0])
            fig.scatter(pltData.xcoord[pltData.res_val==bad],
                        pltData.ycoord[pltData.res_val==bad], marker=m[1], s=s[1], c=c[1], alpha=a[1])
            fig.suptitle(str(pltData.alias_lot_id.unique() + pltData.wf_id.unique()))
            #fig.tight_layout()

        
        return fig
        
